local Sackville = LibStub("AceAddon-3.0"):GetAddon("Sackville", "AceConsole-3.0")
Sackville.Section = {}
Sackville.Section.__index = Sackville.Section

function Sackville.Section:New(parentBag, sectionName, sectionConfig)
    Sackville:Debug(1, "section", "Adding new section: %s", sectionName)
    local self = {}
    setmetatable(self, Sackville.Section)
    
    local section = {}
    local sectionDefaults = Sackville.Options.section
    self.name = sectionName
    self.bagName = parentBag.name
    self.displayOrder = sectionConfig.displayOrder
    if sectionConfig.useDefaultPriority then
        self.priority = parentBag.defaultPriority
    else
        self.priority = sectionConfig.priority
    end
    if sectionConfig.useDefaultSorting then
        self.sortCriteria = parentBag.defaultSortCriteria
    else
        self.sortCriteria = sectionConfig.sortCriteria
    end
    
    self.alwaysMatch = sectionConfig.alwaysMatch or false
    self.categories = sectionConfig.categories or {}
    self.showEmptySlot = sectionConfig.showEmptySlot or false
    self.alwaysShow = sectionConfig.alwaysShow
    self.itemKeys = {}
    self.row = 0
    self.col = 0
    -- self.max_width = parentBag.max_width - parentBag.margin_left - parentBag.margin_right

    return self
end

function Sackville.Section:AddItem(item)
    item:SetBagName(self.bagName)
    item:SetSectionName(self.name)
    item:SetParent(self.frame)
    item:Show()
    
    table.insert(self.itemKeys, item:GetBagSlot())

    Sackville:Debug(2, "section", "AddItem: %s", item:Dump())
end

function Sackville.Section:RemoveItem(item)   
    Sackville:Debug(2, "section", "RemoveItem: %", item:Dump())
    item:SetBagName("")
    item:SetSectionName("")
    item:Hide()
    item:SetParent(Sackville.dummyFrame)
    
    for i=1,#self.itemKeys do
        if self.itemKeys[i] == item:GetBagSlot() then
            table.remove(self.itemKeys, i)
            break
        end
    end
end

function Sackville.Section:HasItems()
    return Sackville.Util.TableHasValues(self.itemKeys)
end

function Sackville.Section:ItemIterator()
    local myItems = {}
    for i, bagslot in ipairs(self.itemKeys) do
        myItems[bagslot] = Sackville.db.char.items[bagslot]
    end
    return Sackville.Util.kpairs(myItems, self.sortCriteria)
end

function Sackville.Section:LayoutItems()
    self.row = 0
    self.col = 0

    -- Position all items in this section
    for _, item in self:ItemIterator() do
        if GetContainerItemID(item.bagid, item.slot) then
            self:PositionItem(item)
        else    -- slot is empty
            item:Hide()
        end
    end

end

function Sackville.Section:MatchItem(item)
-- Returns true if the item matches the categories assigned to this section
    if self.alwaysMatch then
        Sackville:Debug(3, "filter", "* Hit default section: %s", self.name)
        return true
    end
    
    
    for _, catName in ipairs(self.categories) do
        local matchRules = Sackville.Options.categories[catName]

        if matchRules and #matchRules > 0 then
            local isMatch = Sackville.Filter:ItemMatchesRules(item, matchRules)
            if isMatch then return true end
        else
            Sackville:Debug(4, "filter", "* Found no rules in section: %s", self.name)        
        end
    end
    
    return false    
end

function Sackville.Section:HighlightSearchItems(text)
    text = string.lower(text)
    for _, item in self:ItemIterator() do
        if text == "" then
            item.button:SetAlpha(1)
        elseif string.find(string.lower(item.name), text) then
            item.button:SetAlpha(1)
        else
            item.button:SetAlpha(0.25)
        end
    end
end

function Sackville.Section:UpdateEmptySlot(bagType)            
    if not self.showEmptySlot then return nil; end
    local emptyCount = Sackville:GetEmptySlotCount(bagType)
    if emptyCount == 0 then return end
    
    local bagSet = nil   
      
    local emptyItem = Sackville.db.char.emptyItem[bagType]
    
    for bagslot, item in pairs(Sackville.db.char.items) do
        if item:IsEmpty() and Sackville:IsBagType(bagType, item:GetBagId()) then
            if emptyItem == nil or not emptyItem.empty then
                emptyItem = item
            else
                item:Hide()
            end
        end
    end

    if emptyItem then
        emptyItem:SetButtonCount(Sackville:GetEmptySlotCount(bagType))
        Sackville.db.char.emptyItem[bagType] = emptyItem
    end
    return emptyItem
end

function Sackville.Section:Show()
    self.frame:Show()
end

function Sackville.Section:Hide()
    self.frame:Hide()
end

function Sackville.Section:GetName()
    return self.name
end

function Sackville.Section:GetWidth()
    return self.frame:GetWidth()
end

function Sackville.Section:GetHeight()
    return self.frame:GetHeight()
end

function Sackville.Section:SetMaxWidth(width)
    self.max_width = width
end

function Sackville.Section:SetPoint(...)
    return self.frame:SetPoint(...)
end
